@@ -53,50 +53,60 @@ class Bearing:
 
 class Distance:
 	'A distance between two points.'
-	def __init__ (self, value, unit = 'meters'):
-		if unit == 'feet':
+	def __init__ (self, value, units = 'meters'):
+		if units in ('feet', 'ft'):
 			self.value = value * 1609.344 / 5280.
-		elif unit == 'miles':
+		elif units in ('miles', 'mi'):
 			self.value = value * 1609.344
-		elif unit == 'meters':
+		elif units in ('meters', 'm'):
 			self.value = value
-		elif unit == 'kilometers':
+		elif units in ('kilometers', 'km'):
 			self.value = value * 1000.
-		elif unit == 'nauticalmiles':
+		elif units == 'nauticalmiles':
 			self.value = value * 1852.
 		else:
-			raise AssertionError ('invalid length unit %s' % unit)
-	def GetValue (self, unit = 'meters'):
-		if unit in ('feet', 'ft'):
+			raise AssertionError ('invalid length unit %s' % units)
+	def GetValue (self, units = 'meters'):
+		if units in ('feet', 'ft'):
 			return self.value / 1609.344 * 5280.
-		elif unit == 'miles':
+		elif units == 'miles':
 			return self.value / 1609.344
-		elif unit in ('meters', 'm'):
+		elif units in ('meters', 'm'):
 			return self.value
-		elif unit in ('kilometers', 'km'):
+		elif units in ('kilometers', 'km'):
 			return self.value / 1000.
-		elif unit == 'nauticalmiles':
+		elif units == 'nauticalmiles':
 			return self.value / 1852.
 		else:
-			raise AssertionError ('invalid length unit %s' % unit)
+			raise AssertionError ('invalid length unit %s' % units)
+	def __call__ (self, units = 'meters'):
+		return self.GetValue (units)
 	def __repr__ (self):
 		return 'Distance (%f, "meters")' % self.value
 
-class ZCommand:
+class ZCommand (object):
 	'A command usable on a character, item, zone, etc. Included in ZCharacter.Commands table.'
 	def __init__ (self, arg):
-		self.Text = arg['Text']
-		self.EmptyTargetListText = arg['EmptyTargetListText']
-		self.Enabled = arg['Enabled']
-		self.CmdWith = arg['CmdWith']
-	def __repr__ (self):
+		self.Text = arg['Text'] if 'Text' in arg else 'None set'
+		self.EmptyTargetListText = arg['EmptyTargetListText'] if 'EmptyTargetListText' in arg else 'None set'
+		self.Enabled = arg['Enabled'] if 'Enabled' in arg else True
+		self.CmdWith = arg['CmdWith'] if 'CmdWith' in arg else False
+		self.WorksWithAll = arg['WorksWithAll'] if 'WorksWithAll' in arg else False
+		self.WorksWithList = arg['WorksWithList'] if 'WorksWithList' in arg else []
+	def x__getattribute__ (self, key):
+		k = 'Get' + key
+		obj = super (ZCommand, self)
+		if hasattr (obj, k):
+			return getattr (obj, k) ()
+		else:
+			return getattr (obj, key)
+	def x__repr__ (self):
 		return '<ZCommand\n\t' + '\n\t'.join (['%s:%s' % (x, str (getattr (self, x))) for x in dir (self) if not x.startswith ('_')]) + '\n>'
 
 class ZObject (object):
-	def __init__ (self, cartridge, container = None):
+	def __init__ (self, Cartridge, Container = None):
 		self.Active = True
-		self.Cartridge = cartridge
-		self.Container = container
+		self.Container = Container
 		self.Commands = []
 		self.CommandsArray = []
 		self.CurrentBearing = Bearing (0)
@@ -105,12 +115,43 @@ class ZObject (object):
 		self.Icon = None
 		self.Id = None
 		self.Inventory = []
+		self.Locked = False
 		self.Media = None
 		self.Name = '[Name for this object is not set]'
 		self.ObjectLocation = INVALID_ZONEPOINT
-		self.ObjIndex = len (cartridge.AllZObjects) + 1
 		self.Visible = True
-		cartridge.AllZObjects += (self,)
+		if isinstance (Cartridge, ZCartridge):
+			self.Cartridge = Cartridge
+		else:
+			for k in Cartridge:
+				setattr (self, k, Cartridge[k])
+			assert isinstance (self.Cartridge, ZCartridge)
+		self.ObjIndex = len (self.Cartridge.AllZObjects) + 1
+		self.Cartridge.AllZObjects += (self,)
+	def Contains (self, obj):
+		return obj.Container == self
+	def MoveTo (self, owner):
+		self.Container = owner
+	def _is_visible (self, debug):
+		if not (debug or (self.Active and self.Visible)):
+			return False
+		if self.Container == None:
+			return False
+		if self.Container == Player:
+			return True
+		if not self.Container.Active or not isinstance (self.Container, Zone):
+			return False
+		if self.Container.ShowObjects == 'OnEnter':
+			if self.Container.State != 'Inside':
+				return False
+		elif self.Container.ShowObjects == 'OnProximity':
+			if self.Container.State not in ('Inside', 'Proximity'):
+				return False
+		elif self.Container.ShowObjects == 'Always':
+			return True
+		else:
+			print ('invalid (or at least unknown) value for ShowObjects: %s' % self.Container.ShowObjects)
+		return True
 	def _show (self):
 		return '<ZObject\n\t' + '\n\t'.join (['%s:%s' % (x, str (getattr (self, x))) for x in dir (self) if not x.startswith ('_')]) + '\n>'
 
@@ -184,23 +225,18 @@ class ZCartridge (ZObject):
 		global Player
 		ZCartridge._instance = None
 		Player = None
-	def _setup (self, cart, cbs, script):
+	def _common_setup (self, cart, script, code):
 		global Player
 		self.Activity = cart.gametype
 		self.Author = cart.author
 		self.Description = cart.description
-		self.Icon.Id = cart.iconId
 		self.Id = cart.guid
-		self.Media.Id = cart.splashId
 		self.Name = cart.name
 		self.StartingLocation = ZonePoint (cart.latitude, cart.longitude, cart.altitude)
 		self.StartingLocationDescription = cart.startdesc
 		self.TargetDevice = cart.device
 		self.Version = cart.version
 		self._mediacount = 1
-		self._image = {}
-		self._sound = {}
-		self._cb = cbs
 		# According to the wiki, both the global "Player" and "wherigo.Player" should be a reference to the current player.
 		Player = ZCharacter (self)
 		# Player should not be in the list, and it should have a negative index.
@@ -208,15 +244,34 @@ class ZCartridge (ZObject):
 		Player.Name = cart.user
 		Player.CompletionCode = cart.completion_code
 		# For technical reasons, the python value wherigo.Player is not available in lua without the statement below.
-		script.run ('Wherigo.Player = Player', 'Player', Player, 'setting Player variables')
+		script.run ('Wherigo.Player = Player', 'Player', Player, name = 'setting Player variables')
+		self.AllZObjects.pop ()
+		# Create a starting marker object, which can be used for drawing a marker on the map, but which is invisible for the cartridge.
+		ret = script.run (code, name = 'cartridge setup')[0]
+		return ret
+	def _setup (self, cart, cbs, script):
+		self.Icon.Id = cart.iconId
+		self.Media.Id = cart.splashId
+		self._image = {}
+		self._sound = {}
+		self._cb = cbs
+		ret = self._common_setup (cart, script, cart.data[0])
+		global _starting_marker
+		_starting_marker = ZItem (self)
+		_starting_marker.ObjectLocation = self.StartingLocation
+		_starting_marker.Name = 'The start of this cartridge'
+		_starting_marker.Media = self.Icon
+		_starting_marker.Description = self.StartingLocationDescription
 		self.AllZObjects.pop ()
-		ret = script.run (cart.data[0])[0]
 		# Set up media.
 		for i in self.AllZObjects:
 			if not isinstance (i, ZMedia) or i._id < 1:
 				continue
-			r = lua.as_list (i.Resources)[0]
-			if r['Type'] in ('wav', 'mp3'):
+			r = i.Resources._list ()
+			if len (r) < 1:
+				continue
+			r = r[0]
+			if r['Type'] in ('wav', 'mp3', 'fdl'):
 				self._sound[i.Id] = cart.data[i._id]
 			else:
 				px = gtk.gdk.PixbufLoader ()
@@ -224,11 +279,46 @@ class ZCartridge (ZObject):
 				px.close ()
 				self._image[i.Id] = px.get_pixbuf ()
 		return ret
+	def _getmedia (self, cart, script, code):
+		class nocb:
+			def dialog (self, table):
+				pass
+			def message (self, table):
+				pass
+			def get_input (self, zinput):
+				pass
+			def play (self, media):
+				pass
+			def stop_sound (self):
+				pass
+			def set_status (self, text):
+				pass
+			def save (self):
+				pass
+			def quit (self):
+				pass
+			def drive_to (self, *a):
+				pass
+			def alert (self):
+				pass
+			def log (self, level, levelname, text):
+				pass
+			def show (self, screen, item):
+				pass
+			def update (self):
+				pass
+			def update_stats (self):
+				pass
+			def update_map (self):
+				pass
+		self._cb = nocb ()
+		self._common_setup (cart, script, code)
+		return [x for x in self.AllZObjects if isinstance (x, ZMedia) and x._id > 0]
 
 class ZCharacter (ZObject):
-	def __init__ (self, cartridge):
+	def __init__ (self, Cartridge):
 		#print 'making character'
-		ZObject.__init__ (self, cartridge)
+		ZObject.__init__ (self, Cartridge)
 		self.name = 'Unnamed character'
 		self.InsideOfZones = []
 		self.Inventory = []
@@ -239,8 +329,8 @@ class ZCharacter (ZObject):
 class ZTimer (ZObject):
 	'A timer object allowing time or activity tracking.'
 	# attributes: Type ('Countdown'|'Interval'), Duration (Number), Id, Name, Visible
-	def __init__ (self, cartridge):
-		ZObject.__init__ (self, cartridge)
+	def __init__ (self, Cartridge):
+		ZObject.__init__ (self, Cartridge)
 		self.Type = 'Countdown'
 		self.Duration = -1
 		self.Remaining = -1
@@ -288,37 +378,55 @@ class ZTimer (ZObject):
 
 class ZInput (ZObject):
 	'A user input field.'
-	def __init__ (self, cartridge):
-		ZObject.__init__ (self, cartridge)
+	def __init__ (self, Cartridge):
+		ZObject.__init__ (self, Cartridge)
 
 class ZItem (ZObject):
 	'An item which can be placed in a zone or held by a character.'
-	def __init__ (self, arg):
-		ZObject.__init__ (self, arg.Cartridge, arg.Container)
+	def __init__ (self, Cartridge, Container = None):
+		if not isinstance (Cartridge, ZCartridge):
+			Cartridge, Container = Cartridge['Cartridge'], Cartridge['Container']
+		ZObject.__init__ (self, Cartridge, Container)
 
 class Zone (ZObject):
 	'Geographical area defined by several ZonePoints.'
-	def __init__ (self, cartridge):
-		ZObject.__init__ (self, cartridge)
+	def __init__ (self, Cartridge):
+		if isinstance (Cartridge, ZCartridge):
+			ZObject.__init__ (self, Cartridge)
+			self.OriginalPoint = INVALID_ZONEPOINT
+			self.ShowObjects = 'OnEnter'
+			self.State = 'NotInRange'
+			self.Inside = False
+		else:
+			assert 'Cartridge' in Cartridge
+			ZObject.__init__ (self, Cartridge['Cartridge'])
+			for k in Cartridge:
+				assert k[0] != '_'
+				if k == 'Cartridge':
+					continue
+				setattr (self, k, Cartridge[k])
+			if not hasattr (self, 'Inside'):
+				self.Inside = False
+			if not hasattr (self, 'State'):
+				self.State = 'NotInRange'
 		self._inside = False
 		self._active = True
 		self._state = 'NotInRange'
-		self.OriginalPoint = INVALID_ZONEPOINT
 	def __str__ (self):
 		return '<Zone at %s>' % str (self.OriginalPoint)
 
 class ZTask (ZObject):
 	'A task the user can attempt to accomplish.'
-	def __init__ (self, cartridge):
-		ZObject.__init__ (self, cartridge)
+	def __init__ (self, Cartridge):
+		ZObject.__init__ (self, Cartridge)
 
 class ZMedia (ZObject):
 	'A media file such as an image or sound.'
-	def __init__ (self, cartridge):
-		ZObject.__init__ (self, cartridge)
-		self._id = cartridge._mediacount
-		if cartridge._mediacount > 0:
-			cartridge._mediacount += 1
+	def __init__ (self, Cartridge):
+		ZObject.__init__ (self, Cartridge)
+		self._id = Cartridge._mediacount
+		if Cartridge._mediacount > 0:
+			Cartridge._mediacount += 1
 
 # These functions are called from lua to make the application do things.
 def Dialog (table):
@@ -373,7 +481,6 @@ def ShowScreen (screen, item = None):
 # These functions seem to be for doing dirty work which is too slow or annoying in lua...
 def NoCaseEquals (s1, s2):
 	'Compares two strings for equality, ignoring case. Uncertain parameters.'
-	print sys._getframe().f_code.co_name
 	return s1.lower () == s2.lower ()
 
 def Inject ():
@@ -381,20 +488,21 @@ def Inject ():
 	print sys._getframe().f_code.co_name
 	pass
 
-def _intersect (point, segment):
+def _intersect (point, segment, name = ''):
 	'Compute whether a line from the north pole to point intersects with the segment. Return 0 or 1.'
+	# Use simple interpolation for latitude. TODO: this is not correct on the spherical surface.
 	lon1 = segment[0].longitude
 	lon2 = segment[1].longitude
-	# Sort lon1 and lon2, so the shortest curve is used.
-	if (lon2 - lon1) % 360 > 180:
-		lon1 = segment[1].longitude
-		lon2 = segment[0].longitude
-	d = lon2 - lon1
 	lonp = point.longitude
-	if (lonp - lon1) % 360 > 180 or (lon1 + d - lonp) % 360 > 180:
-		return 0
-	# Use simple interpolation. TODO: this is not correct on the spherical surface.
-	lat = segment[0].latitude + (segment[1].latitude - segment[0].latitude) * ((lonp - lon1) % 360) / d
+	if (lon2 - lon1) % 360 > 180:
+		# lon1 > lon2
+		if (lonp - lon2) % 360 > 180 or (lon1 - lonp) % 360 >= 180 or lon1 == lonp:
+			return 0
+		lat = segment[0].latitude + (segment[1].latitude - segment[0].latitude) * ((lonp - lon2) % 360) / ((lon1 - lon2) % 360)
+	else:
+		if (lonp - lon1) % 360 > 180 or (lon2 - lonp) % 360 >= 180 or lon2 == lonp:
+			return 0
+		lat = segment[0].latitude + (segment[1].latitude - segment[0].latitude) * ((lonp - lon1) % 360) / ((lon2 - lon1) % 360)
 	if lat > point.latitude:
 		return 1
 	return 0
@@ -405,11 +513,11 @@ def IsPointInZone (point, zone):
 	# This means that any line from OriginalPoint to point has an even number of intersections with zone segments.
 	# This line doesn't need to be the shortest path. It is much easier if it isn't. I'm using a two-segment line: One segment straight north to the pole, one straight south to OriginalPoint.
 	num = 0
-	points = lua.as_list (zone.Points)
+	points = zone.Points._list ()
 	points += (points[0],)
 	for i in range (len (points) - 1):
-		num += _intersect (point, (points[i], points[i + 1]))
-		num += _intersect (zone.OriginalPoint, (points[i], points[i + 1]))
+		num += _intersect (point, (points[i], points[i + 1], zone.Name))
+		num += _intersect (zone.OriginalPoint, (points[i], points[i + 1], zone.Name))
 	return num % 2 == 0
 
 def VectorToSegment (point, p1, p2):
@@ -433,7 +541,7 @@ def VectorToZone (point, zone):
 	if IsPointInZone (point, zone):
 		return Distance (0), Bearing (0)
 	# Use VectorToSegment multiple times.
-	points = lua.as_list (zone.Points)
+	points = zone.Points._list ()
 	current = VectorToSegment (point, points[-1], points[0])
 	for p in range (1, len (points)):
 		this = VectorToSegment (point, points[p - 1], points[p])
